# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import GeneratorPlugin, Layers
from rzn.api.units import mm, inch
from rzn.api.common import addrect

class HeaderPlugin(GeneratorPlugin):
  def __init__(self):
    GeneratorPlugin.__init__(self, "Header", [
      ("Column count", "ccount",  int, 16),
      ("Row count",    "rcount",  int, 2),
      ("Pitch (inch)",   "pitch",   float, 0.1),
    ])

  def footprint(self, project, options):
    rows, cols, pitch = options["rcount"], options["ccount"], options["pitch"]
    pitch *= inch(1)
    bodywidth = rows*pitch
    bodyheight = cols*pitch
    po = ((cols-1)*pitch)/2.0
    project.grid(1)

    diam = pitch*0.63
    drill = pitch*0.4

    # add a rectangular pad
    def addpad(n, x, y):
      shp = "rectangle" if n == 0 else "circle"
      p=project.pad((x,y), diam, diam, shp, drill=drill)
      p.name = "%d" % (n+1)

    # add pads
    if rows == 1:
      for i in range(cols):
        addpad(i, 0, i*pitch-po)
    elif rows == 2:
      for i in range(cols):
        addpad(i*2,     -pitch/2., i*pitch-po)
        addpad(i*2 + 1,  pitch/2., i*pitch-po)
    else:
      raise Exception("Only support 1 or 2 rows")

    # add body outline with corner
    addrect(project.layout.line, bodywidth/2.0, bodyheight/2.0, mm(0.25), 
            Layers.OutlineTop, cs=bodywidth/4.0)
    # name
    project.layout.text((-bodywidth/2.0,-bodyheight/2.0-mm(0.1)),
                        "$name", 1, Layers.NameTop)

  def symbol(self, project, options):
    rows, cols = options["rcount"], options["ccount"]
    height = (cols+1)*2
    width = (rows)*2
    po = (height/2.0 - 2)
    project.grid(inch(0.05))
    
    # add a pin
    def addpin(n, x, y):
      p = project.schematic.pin((x,y))
      p.name = "%d" % (n+1)
      p.angle = 180 if x <= 0 else 0

    # add pins
    if rows == 1:
      for i in range(cols):
        addpin(i,       -width/2-2, i*2-po)
    elif rows == 2:
      for i in range(cols):
        addpin(i*2,       -width/2-2, i*2-po)
        addpin(i*2 + 1,    width/2+2, i*2-po)

    # add body outline
    addrect(project.schematic.line, width/2.0, height/2.0, 0, Layers.Symbol)
    # name
    project.schematic.text((-width/2.0,-height/2.0), "$name", 1, Layers.Label)
