# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import GeneratorPlugin, Layers
from rzn.api.units import mm, inch
from rzn.api.common import addrect

def addfoursides(pps, po, pitch, addpin):
  def pi(i): return (i-((pps-1)/2.0))*pitch

  for i in range(pps): addpin(i,       -po,     pi(i), False)
  for i in range(pps): addpin(pps+i,    pi(i),  po   , True)
  for i in range(pps): addpin(pps*2+i,  po,    -pi(i), False)
  for i in range(pps): addpin(pps*3+i, -pi(i), -po   , True)


class QFPPlugin(GeneratorPlugin):
  def __init__(self):
    GeneratorPlugin.__init__(self, "QFP", [
      ("Pin count",       "count",  int, 32),
      ("Pin pitch (mm)",  "pitch",  float, 0.8),
      ("Pin width (mm)",  "width",  float, 0.4),
      ("Pin length (mm)", "length", float, 1.5),
    ])

  def footprint(self, project, options):
    pins, pitch, pinwidth, pinheight = (options["count"], options["pitch"],
                                        options["width"], options["length"])
    pitch *= mm(1)
    pinwidth *= mm(1)
    pinheight *= mm(1)
    pps = pins / 4 # pins per side
    bodysize = (pps) * pitch + mm(0.6) # size of the physical package
    pinofs = pinwidth*1.5 # offset of a pin from origin
    po = (bodysize/2.0 + pinofs)
    project.grid(1)

    # add a rectangular pad
    def addpad(n, x, y, vert=True):
      w, h = (pinwidth, pinheight) if vert else (pinheight, pinwidth)
      p=project.layout.pad((x,y), w, h, "rectangle")
      p.name = "%d" % (n+1)

    # add pads
    addfoursides(pps, po, pitch, addpad)
    # add body outline with corner
    addrect(project.layout.line, bodysize/2.0, bodysize/2.0, mm(0.25),
            Layers.OutlineTop, cs=inch(0.05))
    # name
    project.layout.text((+bodysize/2.0,-bodysize/2.0-mm(0.1)),
                        "$name", 1, Layers.NameTop)

  def symbol(self, project, options):
    pins = options["count"]
    pps = pins / 4 # pins per side
    bodysize = (pps+1)*2
    pinofs = 2
    po = (bodysize/2.0 + pinofs)
    project.grid(inch(0.05))
    
    # add a pin
    def addpin(n, x, y, vert):
      p = project.schematic.pin((x,y))
      p.name = "%d" % (n+1)
      if vert:
        p.angle = 270 if y < 0 else 90
      else:
        p.angle = 180 if x < 0 else 0

    # add pins
    addfoursides(pps, po, 2, addpin)
    # add body outline
    addrect(project.schematic.wire, bodysize/2.0, bodysize/2.0, 0, 
            Layers.Symbol) 
    # name
    project.schematic.text((bodysize/2.0,bodysize/2.0+1), "$name", 1, 
                           Layers.Label)
