# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import GeneratorPlugin, Layers
from rzn.api.units import mm, inch
from rzn.api.common import addrect

class SMDRCLPlugin(GeneratorPlugin):
  def __init__(self):
    GeneratorPlugin.__init__(self, "SMD Passive", [
      ("Pad width (mm)",   "padw",  float, 1.1),
      ("Pad height (mm)",  "padh",  float, 2.0),
      ("Pad spacing (mm)", "space", float, 0.5),
    ])

  def footprint(self, project, options):
    padw, padh, padspace = options["padw"], options["padh"], options["space"]
    project.grid(1)
    padw *= mm(1)
    padh *= mm(1)
    padspace *= mm(1)

    # add pads
    padofs = (padw+padspace)/2.0
    p=project.pad((padofs,0), padw, padh, "rectangle")
    p.name = "a"
    p=project.pad((-padofs,0), padw, padh, "rectangle")
    p.name = "b"

    bodyw = padw*2 + padspace + mm(0.5)
    bodyh = padh + mm(0.5)

    # add body outline
    addrect(project.layout.line, bodyw/2.0, bodyh/2.0, mm(0.25), Layers.OutlineTop)
    # name
    project.layout.text((-bodyw/2.0,-bodyh/2.0-mm(0.1)), "$name", 1, Layers.NameTop)

  def symbol(self, project, options):
    project.grid(inch(0.05))
    
    # add pins
    p = project.schematic.pin((5,0))
    p.name = "a"
    p.showname = False
    p = project.schematic.pin((-5,0))
    p.name = "b"
    p.angle = 180
    p.showname = False
    # add body outline
    addrect(project.schematic.line, 3, 1, 0, Layers.Symbol)
    # name
    project.schematic.text((-3,-1), "$name", 1, Layers.Label)
    # value
    project.schematic.text((-3,2), "$value", 1, Layers.Label)
