#!/usr/bin/env python
# vim:set ts=2 sw=2 sts=2 et:

import os
import sys
sys.path.append(os.path.join(os.path.dirname(sys.argv[0]), "bin"))
# do this before we start logging to hide the PyOpenGL import errors in log
import rzn.glui
from rzn.ui import mainapp
from rzn.log import Logger
import traceback
import signal
import code

log = Logger(__name__)

def handle_exception(exc_type, exc_value, exc_traceback):
  # FIXME: can't seem to avoid these on Mac, but ignoring seems to work
  if exc_type.__name__ == "GLError" and exc_value.err == 1286:
    log.error("Ignoring GL framebuffer exception")
    return
  mainapp.unhandledExceptionDialog(exc_type, exc_value, exc_traceback)
  sys.exit()

def debug(sig, frame):
    d={'_frame':frame}         # Allow access to frame object.
    d.update(frame.f_globals)  # Unless shadowed by global
    d.update(frame.f_locals)

    i = code.InteractiveConsole(d)
    message  = "Signal recieved : entering python shell.\nTraceback:\n"
    message += ''.join(traceback.format_stack(frame))
    i.interact(message)

def main():
  sys.excepthook = handle_exception
  if sys.platform == "linux2":
    signal.signal(signal.SIGUSR1, debug)
  sys.exit(mainapp.run(sys.argv))

if __name__ == '__main__':
  main()
